#!/usr/bin/env python
# -*- coding: euc-jp -*-

"""
 \file ManagerConfig.py
 \brief RTC manager configuration
 \date $Date: $
 \author Noriaki Ando <n-ando@aist.go.jp> and Shinji Kurihara

 Copyright (C) 2003-2005
     Task-intelligence Research Group,
     Intelligent Systems Research Institute,
     National Institute of
         Advanced Industrial Science and Technology (AIST), Japan
     All rights reserved.
"""


import sys
import os
import re
import getopt
import platform

import OpenRTM

  
  
class ManagerConfig :
	"""
	\if jp

	\class ManagerConfig
	\brief Manager configuration $B%/%i%9(B

	Manager $B$N%3%s%U%#%.%e%l!<%7%g%s$r9T$&!"%3%^%s%I%i%$%s0z?t$r<u$1<h$j!"(B
	($B$"$k$$$O0z?t$J$7$G(B)$B%$%s%9%?%s%92=$5$l$k!#!"(BManager $B$N%W%m%Q%F%#$NA0@_Dj(B
	$B$r9T$$(B

	$B@_Dj(B($B%U%!%$%k(B)$B$N;XDj$N6/$5$O0J2<$N$H$*$j$G$"$k!#(B
	$B>e$,$b$C$H$b6/$/!"2<$,$b$C$H$b<e$$!#(B
	<OL>
	<LI>$B%3%^%s%I%i%$%s%*%W%7%g%s(B "-f"
	<LI>$B4D6-JQ?t(B "RTC_MANAGER_CONFIG"
	<LI>$B%G%U%)%k%H@_Dj%U%!%$%k(B "./rtc.conf"
	<LI>$B%G%U%)%k%H@_Dj%U%!%$%k(B "/etc/rtc.conf"
	<LI>$B%G%U%)%k%H@_Dj%U%!%$%k(B "/etc/rtc/rtc.conf"
	<LI>$B%G%U%)%k%H@_Dj%U%!%$%k(B "/usr/local/etc/rtc.conf"
	<LI>$B%G%U%)%k%H@_Dj%U%!%$%k(B "/usr/local/etc/rtc/rtc.conf"
	<LI>$BKd$a9~$_%3%s%U%#%.%e%l!<%7%g%sCM(B
	</OL>
	$B$?$@$7!"%3%^%s%I%i%$%s%*%W%7%g%s(B "-d" $B$,;XDj$5$l$?>l9g$O!"(B
	($B$?$H$((B -f $B$G@_Dj%U%!%$%k$r;XDj$7$F$b(B)$BKd$a9~$_%3%s%U%#%.%e%l!<%7%g%sCM(B
	$B$,;HMQ$5$l$k!#(B

	\else

	\brief

	\endif
	"""
  
	# The list of default configuration file path.
	config_file_path = ["./rtc.conf",
						"/etc/rtc.conf",
						"/etc/rtc/rtc.conf",
						"/usr/local/etc/rtc.conf",
						"/usr/local/etc/rtc/rtc.conf",
						None]

	# Environment value to specify configuration file
	config_file_env = "RTC_MANAGER_CONFIG"


	def __init__(self, argc=None, argv=None):
		"""
        \if jp

        \brief ManagerConfig $B%3%s%9%H%i%/%?(B

        $BM?$($i$l$?0z?t$K$h$j=i4|2=$bF1;~$K$9$k%3%s%9%H%i%/%?!#(B

        \param argc $B%3%^%s%I%i%$%s0z?t$N?t(B
        \param argv $B%3%^%s%I%i%$%s0z?t(B

        \else

        \brief ManagerConfig constructor

        The constructor that performs initialization at the same time with
        given arguments.

        \param argc The number of command line arguments
        \param argv The command line arguments

        \endif
		"""
		self._configFile = ""
		if argc != None and argv != None:
			self.init(argc,argv)


	def __del__(self):
		"""
        \if jp
        \brief ManagerConfig $B%G%9%H%i%/%?(B
        \else
        \brief ManagerConfig destructor
        \endif
		"""
		pass


	def init(self, argc, argv):
		"""
        \if jp
        \brief $B=i4|2=(B

        $B%3%^%s%I%i%$%s0z?t$rM?$($F=i4|2=$9$k!#%3%^%s%I%i%$%s%*%W%7%g%s$O(B
        $B0J2<$N$b$N$,;HMQ2DG=$G$"$k!#(B

        -f file   : $B%3%s%U%#%.%e%l!<%7%g%s%U%!%$%k$r;XDj$9$k!#(B<br>
        -l module : $B%m!<%I$9$k%b%8%e!<%k$r;XDj$9$k!#(B<br>
        -o options: $B$=$NB>%*%W%7%g%s$r;XDj$9$k!#!#(B<br>
        -d        : $B%G%U%)%k%H$N%3%s%U%#%.%e%l!<%7%g%s$r;H$&!#(B<br>
        \else
        \brief Initialization

        Initialize with command line options. The following command options
        are available.

        -f file   : Specify a configuration file. <br>
        -l module : Specify modules to be loaded at the beginning. <br>
        -o options: Other options. <br>
        -d        : Use default static configuration. <br>
        \endif
		"""
		self.parseArgs(argc, argv)


	def configure(self, prop):
		"""
        \if jp
        \brief Configuration $B$N7k2L$r(BProperty$B$KH?1G$5$;$k(B
        \else
        \brief Apply configuration results to Property
        \endif
		"""
		prop.setDefaults(OpenRTM.default_config)
		if self.findConfigFile():
			try:
				fd = file(self._configFile,"r")
				prop.load(fd)
				fd.close()
			except:
				print "Error: file open."
		return self.setSystemInformation(prop)

	"""
        \if jp

        \brief $B%3%s%U%#%.%e%l!<%7%g%s$r<hF@$9$k(B

        $B%3%s%U%#%.%e%l!<%7%g%s$r<hF@$9$k!#(Binit()$B8F$S=P$7A0$K8F$V$H!"(B
        $B@EE*$KDj5A$5$l$?%G%U%)%k%H$N%3%s%U%#%.%e%l!<%7%g%s$rJV$9!#(B
        init() $B8F$S=P$78e$K8F$V$H!"%3%^%s%I%i%$%s0z?t!"4D6-JQ?tEy$K(B
        $B4p$E$$$?=i4|2=$5$l$?%3%s%U%#%.%e%l!<%7%g%s$rJV$9!#(B

        \else

        \brief Get configuration value.

        This operation returns default configuration statically defined,
        when before calling init() function. When after calling init() function,
        this operation returns initialized configuration value according to
        command option, environment value and so on.

        \endif
    """
	#def getConfig(self):
	#pass


	def parseArgs(self, argc, argv):
		"""
        \if jp
        \brief $B%3%^%s%I0z?t$r%Q!<%9$9$k(B

        -f file   : $B%3%s%U%#%.%e%l!<%7%g%s%U%!%$%k$r;XDj$9$k!#(B<br>
        -l module : $B%m!<%I$9$k%b%8%e!<%k$r;XDj$9$k!#!#(B<br>
        -o options: $B$=$NB>%*%W%7%g%s$r;XDj$9$k!#!#(B<br>
        -d        : $B%G%U%)%k%H$N%3%s%U%#%.%e%l!<%7%g%s$r;H$&!#(B<br>
        \else
        \brief Parse command arguments

        -f file   : Specify a configuration file. <br>
        -l module : Specify modules to be loaded at the beginning. <br>
        -o options: Other options. <br>
        -d        : Use default static configuration. <br>
        \endif
		"""
		try:
			opts, args = getopt.getopt(argv[1:], "f:l:o:d:")
		except getopt.GetoptError:
			print "Error: getopt error!"
			sys.exit(0)

		for opt, arg in opts:
			if opt == "-f":
				self._configFile = arg

			if opt == "-l":
				pass

			if opt == "-o":
				pass

			if opt == "-d":
				pass

		return


	def findConfigFile(self):
		"""
        \if jp
        \brief Configuration file $B$rC5$9(B

        Configuration file $B$NM%@h=g0L(B

        $B%3%^%s%I%*%W%7%g%s;XDj!d4D6-JQ?t!d%G%U%)%k%H%U%!%$%k!d%G%U%)%k%H@_Dj(B

        $B%G%U%)%k%H6/@)%*%W%7%g%s(B(-d): $B%G%U%)%k%H%U%!%$%k$,$"$C$F$bL5;k$7$F(B
            $B%G%U%)%k%H@_Dj$r;H$&(B
        \else
        \brief Find configuration file
        \endif
		"""
		if self._configFile != "":
			if self.fileExist(self._configFile):
				return True

		env = os.getenv(self.config_file_env)
		if env != None:
			if self.fileExist(env):
				self._configFile = env
				return True

		i = 0
		while (self.config_file_path[i] != None):
			if self.fileExist(self.config_file_path[i]):
				self._configFile = self.config_file_path[i]
				return True
			i += 1

		return False


	def setSystemInformation(self, prop):
		"""
        \if jp

        \brief $B%7%9%F%`>pJs$r%;%C%H$9$k(B

        $B%7%9%F%`>pJs$r<hF@$7%W%m%Q%F%#$K%;%C%H$9$k!#@_Dj$5$l$k%-!<$O0J2<$NDL$j!#(B
        manager.os.name    : OS$BL>(B
        manager.os.release : OS$B%j%j!<%9L>(B
        maanger.os.version : OS$B%P!<%8%g%sL>(B
        manager.os.arch    : OS$B%"!<%-%F%/%A%c(B
        manager.os.hostname: $B%[%9%HL>(B
        manager.pid        : $B%W%m%;%9(BID
        \else
        \brief Set system information

        Get the following system info and set them to Manager's properties.
        manager.os.name    : OS name
        manager.os.release : OS release name
        maanger.os.version : OS version
        manager.os.arch    : OS architecture
        manager.os.hostname: Hostname
        manager.pid        : process ID
        \endif
		"""
		sysinfo = platform.uname()

		prop.setProperty("manager.os.name",     sysinfo[0])
		prop.setProperty("manager.os.hostname", sysinfo[1])
		prop.setProperty("manager.os.release",  sysinfo[2])
		prop.setProperty("manager.os.version",  sysinfo[3])
		prop.setProperty("manager.os.arch",     sysinfo[4])
		prop.setProperty("manager.pid",         os.getpid())
		
		return prop


	def fileExist(self, filename):
		"""
        \if jp
        \brief $B%U%!%$%k$,B8:_$9$k$+$I$&$+3N$+$a$k(B
        \else
        \brief Check file existance
        \endif
		"""
		try:
			fp = open(filename)
		except:
			print "Can't open file:", filename
			return False
		else:
			fp.close()
			return True

		return False


